function [v] = argMinH(u,y,p,epsilon,OCP)
%Calculates the minimum of the augmented Hamiltonian in a certain grid
%point with a secant method

analytical_formula=1;

if (OCP.beta==0)    %If there is a smooth cost functional, calculate the minimum directly  
    v=min(max((2*epsilon*u+p*y)/(OCP.alpha+2*epsilon),OCP.u_lo),OCP.u_up);
else                %Perform a secant method to find a minimum of the augmented Hamiltonian
    if(analytical_formula==1)
            u1=min(max(0,(2*epsilon*u+p*y-OCP.beta)/(OCP.alpha+2*epsilon)),OCP.u_up); 
            u2=min(max(OCP.u_lo,(2*epsilon*u+p*y+OCP.beta)/(OCP.alpha+2*epsilon)),0);           
            %Corresponding values of the augmented Hamiltonian
            H1=(OCP.alpha/2)*u1^2+OCP.beta*abs(u1)-u1*p*y+epsilon*((u1-u)^2);
            H2=(OCP.alpha/2)*u2^2+OCP.beta*abs(u2)-u2*p*y+epsilon*((u2-u)^2);
            %Choose the update for the control at the corresponding grid point that has the lowest 
            %value of the augmented Hamiltonian
            [~,pos]=min([H1,H2]);        
            switch pos
                case 1
                    v=u1;
                case 2
                    v=u2;
            end

    else
        num_Int=100;    %Number of intervals K_U is intersected in
        Disc_KU=linspace(OCP.u_lo,OCP.u_up,num_Int);    %Discretise K_U into num_Int intervals
        dist=inf;

         while(dist>10^-12)            %If the interval within the minimum is is smaller than 10^-12 stop
          %Find the position of a minimum on the disretised Disc_KU
          [~,pos]=min((OCP.alpha/2)*Disc_KU.^2+OCP.beta*abs(Disc_KU)-Disc_KU.*y*p+epsilon*((Disc_KU-u).^2));
          %Set the argument that minimised the augmented Hamiltonian as the current value for the control in 
          %the corresponding grid point
          v=Disc_KU(pos); 
          if (pos==1)     %If the interval is on the lower boundary of Disc_KU
            Disc_KU=linspace(Disc_KU(pos),Disc_KU(pos+1),num_Int); %Discretise the interval around the minimum 
            dist=(Disc_KU(pos+1)-Disc_KU(pos));                    %New distance of an interval 
          elseif  (pos==max(size(Disc_KU)))                          %Case if minimum is on the upper bound
            Disc_KU=linspace(Disc_KU(pos-1),Disc_KU(pos),num_Int);
            dist=(Disc_KU(pos)-Disc_KU(pos-1));
          else
            %If the minimum is in the inner of Disc_KU, discretise around the minimum
            Disc_KU=linspace(Disc_KU(pos-1),Disc_KU(pos+1),num_Int);    
            dist=Disc_KU(pos+1)-Disc_KU(pos-1);
          end
          end
    end

end 
end